#####################################################################################################
### Supplemental material for Hoque and Torabi, "Modeling the random effects covariance matrix
### for longitudinal data with covariates measurement error,"
### Statistics in Medicine

### The following codes are provided to assist readers in implementing the data analysis 
### results provided in Table 3 of Application Section (Section 6.1.1).
#####################################################################################################
### R code for parameter estimation under three approaches for Data Analysis.
#####################################################################################################
### Data and variables information.
### Need to call the data.
# Data_MFUS
### Variables extracted from the data.
n<-length(unique(Data_MFUS$id)) # number of individuals.
q<-max(Data_MFUS$followup) # number of follow-ups for each individuals.
y<-Data_MFUS$HBP # response variable
Wij<-Xij<-Data_MFUS$bmi #(error-covariate) 
# error-free covariates        
Zij<-Data_MFUS$px_age
Zistar<-Data_MFUS$IHD_new
Zi=matrix(Zistar,n,q,byrow = T)

######################################################################################################
############################ Approach 1: Naive 1 approach ############################################
## Function for Naive 1 approach:
##########################################################

naive1<-function(n,q)
{
metro<- function(theta,n,q)
{
  ones<-matrix(rep(1,q),q,1) # qx1 vector pf 1's.
  Ustar<-rnorm(n,0,sqrt(theta[5])) # initial simulation of random effects.
  z<-matrix(0,nrow=n*q,ncol=n) # nqxn, Design matrix
  for(j in 1:n)
  {
    z[(1+(j-1)*q):(q*j),j]=1
  }
  
  fixed<-theta[1]+theta[2]*Xij+theta[3]*Zij+theta[4]*Zistar # nx1 fixed effects in linear predictor.
  Fixed<-matrix(rep(fixed,1),n*q,1,byrow=T) # nqx1; copies of fixed effects.
  umat<-matrix(rep(Ustar,1),n,1)# nx1; copies of previously simulated random effects.
  eta<-Fixed+z%*%umat # nqx1, copies of linear predictors based on previously simulated u's.
  pij<-exp(eta)/(1+exp(eta)) # probability of bernoulli trail.
  list(Ustar=Ustar,pij=pij)
}

### Expected-Maximization algorithm steps.
### Function for calculating the E and M-steps of MCEM algorithm.
MCEM<-function(M,tol,theta.o)
{
  # M= Number of MC size.
  # tol= tolerance value for EM algorithm
  # theta.o= Intial values of parameters
  k<-0
  theta.k<-theta.o
  beta<-c(theta.k[1],theta.k[2],theta.k[3],theta.k[4])
  sample<-metro(theta.k,n,q)
  y<-y
  U<-sample$Ustar
  yvector<-matrix(y,ncol=1,byrow=T)
  uvector2<-matrix(U,ncol=1,byrow=T)
 ## Likelihood function for fixed effects estimate 
  ll<-function(beta)
  {
    beta0<-beta[1]
    beta1<-beta[2]
    beta2<-beta[3]
    beta3<-beta[4]
    U<-sample$Ustar
    fixed<-beta0+beta1*Wij+beta2*Zij+beta3*Zistar
    Fixed<-matrix(rep(fixed,1),n*q,1,byrow=T)
    z<-matrix(0,nrow=n*q,ncol=n) # nqxn, Design matrix.
    for(j in 1:n)
    {
      z[(1+(j-1)*q):(q*j),j]=1
    }
    uvector<-matrix(z%*%U,ncol=1,byrow=T)
    eta<-Fixed+uvector
    p<-exp(eta)/(1+exp(eta))
    d<-log(dbinom(yvector,1,p))
    sum(d)
  }
## Likelihood function for random effects estimate  
  varu<-theta.k[5]
  ll.random<-function(varu)
  {
    U<-sample$Ustar
    z<-matrix(0,nrow=n*q,ncol=n) # nqxn, Design matrix.
    for(j in 1:n)
    {
      z[(1+(j-1)*q):(q*j),j]=1
    }
    uvector<-matrix(z%*%U,ncol=1,byrow=T)   
    uvector2<-matrix(U,ncol=1,byrow=T)
    sum(log(dnorm(uvector,0,sqrt(varu))))
  }
## M step of EM algorithm. 
 convergence <- F
  while(1-convergence){
    k <- k+1
# updated estimates for fixed parameters
    beta<-optim(fn = ll, par = c(theta.k[1],theta.k[2],theta.k[3],theta.k[4]),method = "BFGS",control=list(fnscale=-1),hessian = TRUE)
    beta.new<-beta$par
    I.beta<-solve(beta$hessian)
    se.beta<-sqrt(abs(diag(I.beta)))
# updated estimates for random effects parameters   
    varu.new<-optimize(ll.random,interval=c(0,5),maximum=TRUE)$maximum
   
    theta.new<-list(beta.new,varu.new)
    theta.new<-unlist(theta.new)
# Checking the convergence
    if (max(abs(c(theta.new[1]-theta.k[1],theta.new[2]-theta.k[2],theta.new[3]-theta.k[3],theta.new[4]-theta.k[4],theta.new[5]-theta.k[5])))>tol){
      theta.k <-theta.new
      beta<-c(theta.new[1],theta.new[2],theta.new[3],theta.new[4])
      varu<-theta.new[5]
    }
    else convergence <- T
  }
  if(convergence){
    print(paste('Convergence was reached after',k,'iteration(s)'))
    mle=theta.new
    
##### Calculation of standard errors (SE) for parameters
  ### Calculation of standard errors for parameters from "optim" function.
   ## NEGATIVE of the hessian is the "fishers observed information"
    I.beta<-solve(-beta$hessian)
    se.beta<-sqrt(diag(I.beta))   # SE for fixed effects
    se.varu<-sqrt(2*varu^2/(n*q)) # SE for random effects  
    estimate<-c(mle,se.beta,se.varu)
    return(estimate)
  }
  else print(paste('Convergence was not reached'))
}
## Given Initial values for all parameters. (From glmmPQL package)
## Intial values of fixed effects parameters are calculated using the "glmmPQL" package in R.
beta.old<-c(-10.116,0.151,0.086,0.512) 
sigma_sq=2.10 
varu.old<-sigma_sq
theta.old<-c(beta.old,varu.old)
### Implication of MCEM function for the MFUS data.
estimate<-MCEM(1000,0.1,theta.old)
## Calculating 95% confidence Interval.
Est<-c(estimate[1],estimate[2],estimate[3],estimate[4],estimate[5])
se<-c(estimate[6],estimate[7],estimate[8],estimate[9],estimate[10])
LB=c()
UB=c()
for (i in 1:5)
{
  LB[i]=Est[i]-1.96*se[i] 
  UB[i]=Est[i]+1.96*se[i]
}
table_naive1<-data.frame(Est,se,LB,UB)
return(table_naive1)
}

## Implementation of Naive 1 approach: (Section 6.1.1: Table 3: (Estimation of parameters))
set.seed(123)
naive1results=naive1(n,q)
round(naive1results,3)


######################################################################################################
############################ Approach 2: Naive 2 approach ############################################
## Function for Naive 2 approach:
#########################################################
naive2<-function(n,q)
 {
# n= Sample size for data generation
# q= Number for follow-up for each individual
## Metropolis-Hasting (MH) algorithm to generate data from random components as described in Section 4.
## Function for MH.
  metropolis<- function (theta.k,M) # M is the number of MC size.
  {
    Ustar<-matrix(0,nrow= M,ncol=n) # Mxn matrix to be filled with simulated u (random effects).
    xstar<-matrix(0,nrow= M,ncol=n*q)
    ap.x<-rep(0,n*q) # to store the acceptence probability for candidate for u.
    ap.u<-rep(0,n) # to store the acceptence probability for candidate for x.
    ones<-matrix(rep(1,q),q,1) # qx1 vector pf 1's.
    Ustar[1,]<-rnorm(n,0,sqrt(theta.k[5])) # initial simulation of random effects.
    ## Candidate distribution for error covariate:X
    xstar[1,]<-rnorm(n*q,theta.k[6],sqrt(theta.k[7]))
    z<-matrix(0,nrow=n*q,ncol=n) # nqxn, Design matrix
    for(j in 1:n)
    {
      z[(1+(j-1)*q):(q*j),j]=1
    }
   ## Metropolis step.
    for (m in 2:M)
    {
      u.candidate<-rnorm(n,0,sqrt(theta.k[5])) #Generate candidate vector for u.
      x.candidate<-matrix(rnorm(n*q,theta.k[6],sqrt(theta.k[7])),n,q)
      xmat<-matrix(xstar[m-1,],n,q) # nxq; copies of previously simulated x values.
      umat<-matrix(rep(Ustar[m-1,],1),n,1)# nx1; copies of previously simulated random effects.
      fixed.denom<-theta.k[1]+theta.k[2]*xmat+theta.k[3]*Zij+theta.k[4]*Zistar
      Fixed.denom<-matrix(rep(fixed.denom,1),n*q,1,byrow=T) # nqx1; copies of ixed effects.
      eta.denom<-Fixed.denom+z%*%umat # nqx1, copies of linear predictors based on previously simulated u's.
      pij.denom<-exp(eta.denom)/(1+exp(eta.denom)) # probability of bernoulli trail.

      y.denom<-y
      Y.denom<-matrix(rep(y.denom,1),n,q)
      
      density.y.denom<-dbinom(Y.denom,1,pij.denom) # nqx1; Binomial density at the linear predictor.
      density.Y.denom<-matrix(density.y.denom,n,q) # nxq; q copies of Binomial density.
      density.X.denom<-matrix(density.y.denom,n,q)
      
      apdenominator.u<-density.Y.denom%*%ones
      apdenominator.x<-density.X.denom

    ## Acceptance vector for random effects,u.

      umat2<-u.candidate # nx1; elements with new candidates.
      fixed.num<-theta.k[1]+theta.k[2]*xmat+theta.k[3]*Zij+theta.k[4]*Zistar # nqx1 fixed effects in linear predictor.
      Fixed.num<-matrix(rep(fixed.num,1),n*q,1,byrow=T) # nqx1; copies of fixed effects.
      eta.num<-Fixed.num+z%*%umat2 # nqx1; copies of linear predictors based on previously simulated u's.
      pij.num<-exp(eta.num)/(1+exp(eta.num))

      y.num<-y
      Y.num<-matrix(rep(y.num,1),n,q)
      density.y.num<-dbinom(Y.num,1,pij.num) # nqx1; jth column has Binomial pmfs with all u's equal 
      # to previously simulated values, except with uj replaced by new candidate. 
      density.Y.num<-matrix(density.y.num,n,q) # nxq; q copies of Binomial density.
      apnumerator.u<-density.Y.num%*%ones # nx1; jth element is the product of n Binomial pmfs at previously simulatated values of u,except with uj  replaced by new candidate.

    ## Acceptance vector for x.  
      xmat2<-x.candidate # nqx1; elements with new candidates.
      fixed.num<-theta.k[1]+theta.k[2]*xmat2+theta.k[3]*Zij+theta.k[4]*Zistar # nx1 fixed effects in linear predictor.
      Fixed.num<-matrix(rep(fixed.num,1),n*q,1,byrow=T) # nqx1; copies of fixed effects.
      
      eta.num<-Fixed.num+z%*%umat # nqx1; copies of linear predictors based on previously simulated u's.
      pij.num<-exp(eta.num)/(1+exp(eta.num))
      
      y.num<-y
      Y.num<-matrix(rep(y.num,1),n,q)
      density.y.num<-dbinom(Y.num,1,pij.num) # nqx1; jth column has Binomial pmfs with all u's equal 
      # to previously simulated values, except with uj replaced by new candidate.
      
      density.Y.num<-matrix(density.y.num,n,q) # nxq; q copies of Binomial density.
      apnumerator.x<-density.Y.num

    ## Acceptance criterion for u and x.     
      ap.u<-ifelse(apnumerator.u<apdenominator.u,apnumerator.u/apdenominator.u,1)
      #acceptance probability of jth element for candidate uj.
      
      ap.x<-ifelse(apnumerator.x<apdenominator.x,apnumerator.x/apdenominator.x,1)
      #acceptance probability of jth element for candidate uj.
      
    ## Acceptance rule.
      # Generate a bernoulli random variable with the candidate u acceptance probabilities.
      ber.u<-rbinom(n,1,ap.u) 
      ber.x<-rbinom(n*q,1,ap.x)
      Ustar[m,]<-ber.u*u.candidate+(1-ber.u)*Ustar[m-1,]  # nx1; new u is either previous u or new candidate.
      xstar[m,]<-ber.x*x.candidate+(1-ber.x)*xstar[m-1,] # nx1; new x is either previous u or new candidate.
    }
    burnin=500
    b=burnin+1
    index=b:M
    Ustar=Ustar[index,]
    xstar=xstar[index,]
    list(xstar=xstar,Ustar=Ustar)
  }
  
### Expected-Maximization algorithm steps.
### Function for calculating the E and M-steps of MCEM algorithm.
  MCEM<-function(M,tol,theta.o)
  { 
  # M= Number of MC size.
  # tol= tolerance value for EM algorithm
  # theta.o= Intial values of parameters
    burnin=500
    Mm<-M-burnin
    k<-0
    theta.k<-theta.o
    beta<-c(theta.k[1],theta.k[2],theta.k[3],theta.k[4]) #fixed effects parameters
    varu<-theta.k[5] # random effects 
    parx<-c(theta.k[6],theta.k[7]) # parameters associated with error variable
    vare<-theta.k[8] # parameter for measurement error variance
## E-M step of EM algorithm.
    convergence <- F
    while(1-convergence){
      k <- k+1
      sample<-metropolis(theta.k,M)
      U<-sample$Ustar 
      Xij<-t(sample$xstar)
      z<-matrix(0,nrow=n*q,ncol=n) # nqxn, Design matrix.
      for(j in 1:n)
      {
        z[(1+(j-1)*q):(q*j),j]=1
      }
        
      fixed<-beta[1]+beta[2]*Wij+beta[3]*Zij+beta[4]*Zistar
      Fixed<-matrix(rep(fixed,1),n*q*Mm,1,byrow=T)
      
      uvector<-matrix(z%*%t(U),ncol=1,byrow=T)
      eta<-Fixed+uvector
      p<-exp(eta)/(1+exp(eta))
      yvector<-matrix(rep(y,1),n*q*Mm,1,byrow=T)
      uvector2<-matrix(U,ncol=1,byrow=T) # Mnx1
      
## Likelihood function for fixed effects estimate
      ll<-function(beta)
      {
        beta0<-beta[1]
        beta1<-beta[2]
        beta2<-beta[3]
        beta3<-beta[4]
        fixed<-beta0+beta1*Wij+beta2*Zij+beta3*Zistar
        Fixed<-matrix(rep(fixed,1),n*q*Mm,1,byrow=T)
        z<-matrix(0,nrow=n*q,ncol=n) # nqxn, Design matrix.
        for(j in 1:n)
        {
          z[(1+(j-1)*q):(q*j),j]=1
        }
        uvector<-matrix(z%*%t(U),ncol=1,byrow=T)
        eta<-Fixed+uvector
        p<-exp(eta)/(1+exp(eta))
        d<-log(dbinom(yvector,1,p))
        sum(d)/Mm
      }
 ## Likelihood function for random effects estimate     
      ll.random<-function(varu)
      {
        uvector2<-matrix(U,ncol=1,byrow=T)
        z<-matrix(0,nrow=n*q,ncol=n) # nqxn, Design matrix.
        for(j in 1:n)
        {
          z[(1+(j-1)*q):(q*j),j]=1
        }
        uvector<-matrix(z%*%t(U),ncol=1,byrow=T)
        -sum(log(dnorm(uvector2,0,sqrt(varu))))/Mm
      }
  
 ## Likelihood function to estimate parameters associated with error variable.    
      ll.x<-function(parx)
      {
        mux=parx[1]
        varx=parx[2]
        xi<-Xij
        -sum(log(dnorm(xi,mux,sqrt(varx))))/Mm
      }
 ## Likelihood function to estimate measurement error variance     
      ll.error<-function(vare)
      { 
        att.factor<-parx[2]/var(Wij) #Attenuation factor
        vare.init<-parx[2]*(1-att.factor)/att.factor
        set.seed(499)
        eij<-rnorm(n*q,0,vare.init) # eij~N(0,vare)
        -sum(log(dnorm(eij,0,sqrt(vare.init))))
      }

# updated estimates for fixed parameters 
      beta<-optim(fn = ll, par = c(theta.k[1],theta.k[2],theta.k[3],theta.k[4]),control=list(fnscale=-1),hessian=TRUE)
      beta.new<-beta$par
      #I.beta<-solve(beta$hessian)
      #se.beta<-sqrt(abs(diag(I.beta)))

# updated estimates for random parameters       
      varu.new<-nlminb(theta.k[5],ll.random)$par

# updated estimates of parameters associated with error variable      
      truexnew<-optim(fn=ll.x,par =c(theta.k[6],theta.k[7]),hessian = T)
      parx.new<-truexnew$par
      #I.parx<-solve(truexnew$hessian)
      #se.parx<-sqrt(diag(I.parx))

# updated estimates for measurement error variance       
      vare.new<-nlminb(theta.k[8],ll.error)$par
## Updated estimates of all parameters.      
      theta.new<-list(beta.new,varu.new,parx.new,vare.new)
      theta.new<-unlist(theta.new)

## Convergence checking      
      if (max(abs(c(theta.new[1]-theta.k[1],theta.new[2]-theta.k[2],theta.new[3]-theta.k[3],theta.new[4]-theta.k[4],theta.new[5]-theta.k[5],theta.new[6]-theta.k[6],theta.new[7]-theta.k[7],theta.new[8]-theta.k[8])))>tol){
        theta.k <-theta.new
        beta<-c(theta.new[1],theta.new[2],theta.new[3],theta.new[4])
        varu<-theta.new[5]
        parx<-c(theta.new[6],theta.new[7])
        vare<-theta.new[8]
      }
      else convergence <- T
    }
    if(convergence){
      print(paste('Convergence was reached after',k,'iteration(s)'))
      mle=theta.new
        
##### Calculation of standard errors (SE) for parameters
  ### Calculation of standard errors for parameters from "optim" function.
   ## NEGATIVE of the hessian is the "fishers observed information"

      I.beta<-solve(-beta$hessian)
      se.beta<-sqrt(diag(I.beta)) # SE for fixed effects
  # SE for parameters associated with error variable: X follows Normal distribution.
      se.mux<- sqrt(parx[2]/(n*q))
      se.varx<-sqrt(2*parx[2]^2/(n*q))      
      se.varu<-sqrt(2*varu^2/(n*q))    # SE for random effects
      se.vare<-sqrt(2*vare^2/(n*q))    # SE for measurement error variance
      
      estimate<-c(mle,se.beta,se.varu,se.mux,se.varx,se.vare)
      return(estimate)
    }
    else print(paste('Convergence was not reached'))
  }
  
## Given Initial values for all parameters. (From glmmPQL package)
## Intial values of fixed effects parameters are calculated using the "glmmPQL" package
  var_Xij<-6.5 #(Initial guess)
  a.factor<-var_Xij/var(Wij) #Attenuation factor
# Initial value for the measurement error variance
  vare.initial<-6.5*(1-a.factor)/a.factor
  theta.old<-c(-10.116,0.151,0.086,0.512,1.953,23.9,6.5,vare.initial)
 
## Implication of MCEM function for the MFUS data.
  estimate<-MCEM(1000,0.1,theta.old)
## Calculating 95% confidence Interval.
  Est<-c(estimate[1],estimate[2],estimate[3],estimate[4],estimate[5],estimate[6],estimate[7],estimate[8])
  se<-c(estimate[9],estimate[10],estimate[11],estimate[12],estimate[13],estimate[14],estimate[15],estimate[16])
  LB=c()
  UB=c()
  for (i in 1:8)
  {
    LB[i]=Est[i]-1.96*se[i]
    UB[i]=Est[i]+1.96*se[i]
  }
  table_naive2<-data.frame(Est,se,LB,UB)
  return(table_naive2)
}

# Implementation of Naive 2 approach: (Section 6.1.1: Table 3: (Estimation of parameters))
set.seed(123)
naive2results=naive2(n,q)
round(naive2results,3)

##########################################################################################################
############################ Approach 3: Proposed approach ###############################################
## Function for Proposed approach:
##########################################################

Proposed<-function(n,q){
# n= Sample size for data generation
# q= Number for follow-up for each individual
## Metropolis-Hasting (MH) algorithm to generate data from random components as described in Section 4.
## Function for MH. 
  metropolis<- function (theta.k,M) # M is the number of MC size.
  {
    Ustar<-matrix(0,nrow=M,ncol=n*q) # matrix to be filled with  simulated u (random effects).
    xstar<-matrix(0,nrow=M,ncol=n*q) # matrix to be filled with simulated x.
    ap.x<-rep(0,n*q) # to store the acceptence probability for candidate for u.
    ap.u<-rep(0,n*q) # to store the acceptence probability for candidate for x.
    ones<-matrix(rep(1,q),q,1) # qx1 vector pf 1's.
## Function for generating random effects by proposed approach following Section 3.
    ucandid<-function(n,q){
    u.all=matrix(NA,n,q)
 # Indicator variable to generate GAR and IV parameters based on information given in Section 6.1.
    I=matrix(NA,q,q)   
    for (i in 1:n){
     for (j in 1:q){
      for (t in (1:q)){
     #I[j,t]=ifelse((abs(j-t)==1),1,0)
     I[j,t]=if(abs(j-t)==1) 1 else 0
      }
       }
    phi=theta.k[5]*I+theta.k[6]*I*Zi[i,]
    T.hat=phi+diag(1,q,q)
    T.hat[upper.tri(T.hat)] <-0
    T.hat[col(T.hat)<row(T.hat)] <- -1*T.hat[col(T.hat)<row(T.hat)]
    sigma_sq=sqrt(exp(theta.k[7]+theta.k[8]*Zi[i,]))
    D.hat=diag(sigma_sq,q,q)
    Sigma=solve(T.hat)%*%D.hat%*%solve(t(T.hat))
    mean=rep(0,q)
 ## Load relevant R library
    #require(mvtnorm)
    library(mvtnorm)
    u.all[i,]=rmvnorm(1,mean,Sigma)
    }
    U.all=matrix(t(u.all),ncol=1,byrow=T)
    return(list(U.all=U.all,u.all=u.all,I=I))
    }

    Ustar[1,]<-ucandid(n,q)$U.all # initial simulation of random effects.
    xstar[1,]<-rnorm(n*q,theta.k[9],sqrt(theta.k[10]))
 ## Metropolis step.
    for (m in 2:M)
    {
      u.candidate<-ucandid(n,q)$U.all #Generate candidate vector for u.
      x.candidate<-matrix(rnorm(n*q,theta.k[9],sqrt(theta.k[10])),n,q) #Generate candidate vector for x.
      xmat<-matrix(xstar[m-1,],n,q) # copies of previously simulated x.
      umat<-matrix(Ustar[m-1,],n,q) # copies of previously simulated random effects.
      umat<-matrix(umat,n*q,1,byrow=T)
      #fixed effects in linear predictor.
      fixed.denom<-theta.k[1]+theta.k[2]*xmat+theta.k[3]*Zij+theta.k[4]*Zistar
      Fixed.denom<-matrix(rep(fixed.denom,1),n*q,1,byrow=T) # nqx1; copies of fixed effects.
      eta.denom<-Fixed.denom+umat # copies of linear predictors based on previously simulated u's.
      pij.denom<-exp(eta.denom)/(1+exp(eta.denom)) # probability of bernoulli trail.
      y.denom<-y
      Y.denom<-matrix(rep(y.denom,1),n,q) # nqx1; copies of the response.
      
      density.y.denom<-dbinom(Y.denom,1,pij.denom) # Binomial density at the linear predictor.
      density.Y.denom<-matrix(density.y.denom,n,q) # nxq; q copies of Binomial density.
      density.X.denom<-matrix(density.y.denom,n,q)
      apdenominator.u<-density.Y.denom # jth element is the product of Binomial pmfs at previously simulatated values of uj. 
      apdenominator.x<-density.X.denom
      
   ## Acceptance vector for random effects,u.

      umat2<-u.candidate # elements with new candidates.
      
      fixed.num<-theta.k[1]+theta.k[2]*xmat+theta.k[3]*Zij+theta.k[4]*Zistar # nx1 fixed effects in linear predictor.
      Fixed.num<-matrix(rep(fixed.num,1),n*q,1,byrow=T) # copies of ixed effects.
      eta.num<-Fixed.num+umat2 # copies of linear predictors based on previously simulated u's.
      pij.num<-exp(eta.num)/(1+exp(eta.num))
      y.num<-y
      Y.num<-matrix(rep(y.num,1),n,q) # copies of the response.
      density.y.num<-dbinom(Y.num,1,pij.num) # nqx1; jth column has Binomial pmfs with all u's equal 
      # to previously simulated values, except with uj replaced by new candidate.
      density.Y.num<-matrix(density.y.num,n,q) # nxq; q copies of Binomial density.
      apnumerator.u<-density.Y.num

   ## Acceptance vector for x.      
      xmat2<-x.candidate # elements with new candidates.
      fixed.num<-theta.k[1]+theta.k[2]*xmat2+theta.k[3]*Zij+theta.k[4]*Zistar # nx1 fixed effects in linear predictor.
      Fixed.num<-matrix(rep(fixed.num,1),n*q,1,byrow=T) # copies of fixed effects.
      eta.num<-Fixed.num+umat # copies of linear predictors based on previously simulated u's.
      pij.num<-exp(eta.num)/(1+exp(eta.num))
 
      y.num<-y
      Y.num<-matrix(rep(y.num,1),n,q) # copies of the response.
      density.y.num<-dbinom(Y.num,1,pij.num) # jth column has Binomial pmfs with all u's equal 
      # to previously simulated values, except with uj replaced by new candidate.
      density.Y.num<-matrix(density.y.num,n,q) # nxq; q copies of Binomial density.
      
      apnumerator.x<-density.Y.num
   
    ## Acceptance criterion for u and x.   
      ap.u<-ifelse(apnumerator.u<apdenominator.u,apnumerator.u/apdenominator.u,1)  
      #acceptance probability of jth element for candidate uj.
      
      ap.x<-ifelse(apnumerator.x<apdenominator.x,apnumerator.x/apdenominator.x,1) 
      #acceptance probability of jth element for candidate uj.
      
    ## Acceptance rule.
      # Generate a bernoulli random variable with the candidate u acceptance probabilities.
      ber.u<-rbinom(n*q,1,ap.u) 
      ber.x<-rbinom(n*q,1,ap.x)
      Ustar[m,]<-ber.u*u.candidate+(1-ber.u)*Ustar[m-1,]  # nx1; new u is either previous u 
      # or new candidate.
      xstar[m,]<-ber.x*x.candidate+(1-ber.x)*xstar[m-1,] # nx1; new x is either previous u 
      # or new candidate.
        }
    burnin=500
    b=burnin+1
    index=b:M
    Ustar=Ustar[index,]
    xstar=xstar[index,]
    list(xstar=xstar,Ustar=Ustar)
  }
  
### Expected-Maximization algorithm steps.
 ## Indicator variable to generate GAR and IV parameters based on information given in Section 6.1.
  I=matrix(NA,q,q)
  for (j in 1:q)
  {
    for (t in (1:q))
    {
      #I[j,t]=ifelse((abs(j-t)==1),1,0)
      I[j,t]=if(abs(j-t)==1) 1 else 0
    }
  }
 ### Function for calculating the E and M-steps of Proposed algorithm.
  MCEM<-function(M,tol,theta.o) # M is the number of MC size.
  { 
  # M= Number of MC size.
  # tol= tolerance value for EM algorithm
  # theta.o= Intial values of parameters
    burnin=500
    Mm<-M-burnin
    k<-0
    theta.k<-theta.o
    beta<-c(theta.k[1],theta.k[2],theta.k[3],theta.k[4]) #fixed effects parameters
    varu<-c(theta.k[5],theta.k[6],theta.k[7],theta.k[8]) # random effects parameters
    parx<-c(theta.k[9],theta.k[10])  # parameters associated with error variable
    vare<-theta.k[11]   # parameter for measurement error variance
  ## E-M step of EM algorithm.     
    convergence <- F
    while(1-convergence){
      k <- k+1
      sample<-metropolis(theta.k,M)
      U<-sample$Ustar 
      Xij<-t(sample$xstar) 
      fixed<-beta[1]+beta[2]*Wij+beta[3]*Zij+beta[4]*Zistar 
      Fixed<-matrix(rep(fixed,1),n*q*Mm,1,byrow=T)
      
      uvector2<-matrix(t(U),ncol=1,byrow=T)
      eta<-Fixed+uvector2
      p<-exp(eta)/(1+exp(eta))
      yvector<-matrix(rep(y,1),n*q*Mm,1,byrow=T)
      
 ## Function for estimating fixed effect parameters.
      ll<-function(beta)
      {
        beta0<-beta[1]
        beta1<-beta[2]
        beta2<-beta[3]
        beta3<-beta[4]
        fixed<-beta0+beta1*Wij+beta2*Zij+beta3*Zistar
        Fixed<-matrix(rep(fixed,1),n*q*Mm,1,byrow=T)
        eta<-Fixed+uvector2
        p<-exp(eta)/(1+exp(eta))
        d<-log(dbinom(yvector,1,p))
        sum(d)/Mm
      }
## Function for estimating random effect parameters.  
      ll.random<-function(varu)
      {
        delta0<-varu[1] # GAR parameters for random effects cov matrix 
        delta1<-varu[2]
        lambda0<-varu[3] # IV parameters for random effects cov matrix
        lambda1<-varu[4]
        u.all=matrix(NA,n,q)
        dd=matrix(NA,n,q)
        logdd=matrix(NA,n,q)
        Sigma=matrix(NA,n,q*q)
        for (i in 1:n){
        phi=delta0*I+delta1*I*Zi[i,]
        T.hat=phi+diag(1,q,q)
        T.hat[upper.tri(T.hat)]<-0
        T.hat[col(T.hat)<row(T.hat)] <- -1*(T.hat[col(T.hat)<row(T.hat)])
        sigma_sq=sqrt(exp(lambda0+lambda1*Zi[i,]))
        D.hat=diag(sigma_sq,q,q)
        Sigma=solve(T.hat)%*%D.hat%*%solve(t(T.hat))
        mean=rep(0,q)
        library(mvtnorm)
        u.all[i,]=rmvnorm(1,mean,Sigma)
        dd=dmvnorm(u.all,mean,Sigma,log=TRUE)
        }
        sum(dd)/n
      }
## Function for estimating true covariates parameters.        
      ll.x<-function(parx)
      {
        mux=parx[1]
        varx=parx[2]
        xi<-Xij
        -sum(log(dnorm(xi,mux,sqrt(varx))))/Mm
      }
## Function for estimating measurement error variance.        
      ll.error<-function(vare)
      { 
        att.factor<-parx[2]/var(Wij) #Attenuation factor
        vare.init<-parx[2]*(1-att.factor)/att.factor
        set.seed(499)
        eij<-rnorm(n*q,0,vare.init) # eij~N(0,vare)
        -sum(log(dnorm(eij,0,sqrt(vare.init))))
      }  
 
 # updated estimates for fixed parameters      
      fixed<-optim(fn = ll, par = c(theta.k[1],theta.k[2],theta.k[3],theta.k[4]),method = "BFGS",control=list(fnscale=-1),hessian = TRUE)
      beta.new<-fixed$par
 # updated estimates for random parameters            
      random<-optim(fn=ll.random, par = c(theta.k[5],theta.k[6],theta.k[7],theta.k[8]),method = "L-BFGS-B",control=list(fnscale=-1),hessian = TRUE)
      varu.new<-random$par

 # updated estimates of parameters associated with error variable     
      truexnew<-optim(fn=ll.x,par =c(theta.k[9],theta.k[10]),hessian = T)
      parx.new<-truexnew$par
      I.parx<-solve(truexnew$hessian)
      se.parx<-sqrt(diag(I.parx))
      
 # updated estimates for measurement error variance m
      vare.new<-nlminb(theta.k[11],ll.error,hessian = T)$par

 ## Updated estimates of all parameters.      
      theta.new<-list(beta.new,varu.new,parx.new,vare.new)
      theta.new<-unlist(theta.new)
      
if (max(abs(c(theta.new[1]-theta.k[1],theta.new[2]-theta.k[2],theta.new[3]-theta.k[3],theta.new[4]-theta.k[4],theta.new[5]-theta.k[5],theta.new[6]-theta.k[6],theta.new[7]-theta.k[7],theta.new[8]-theta.k[8],theta.new[9]-theta.k[9],theta.new[10]-theta.k[10],theta.new[11]-theta.k[11])))>tol){
        theta.k <-theta.new      
        beta<-c(theta.new[1],theta.new[2],theta.new[3],theta.new[4])
        varu<-c(theta.new[5],theta.new[6],theta.new[7],theta.new[8])
        parx<-c(theta.new[9],theta.new[10])
        vare<-theta.new[11]
      }
      else convergence <- T
    }
    if(convergence){
      print(paste('Convergence was reached after',k,'iteration(s)'))
      mle=theta.new         

### Calculation of standard errors for parameters from "optim" function.
   # NEGATIVE of the hessian is the "fishers observed information"
    I.fixed<-solve(-fixed$hessian) 
    se.fixed<-sqrt(diag(I.fixed)) # SE for fixed effects

    I.rand<-solve(-random$hessian)
    se.rand<-sqrt(abs(diag(I.rand))) # SE for random effects parameters

# SE for parameters associated with error variable: X follows Normal distribution.
    se.mux<- sqrt(parx[2]/(n*q))  
    se.varx<-sqrt(2*parx[2]^2/(n*q)) 
    se.vare<-sqrt(2*vare^2/(n*q)) # SE for measurement error variance
    estimate<-c(mle,se.fixed,se.rand,se.mux,se.varx,se.vare)
    return(estimate)
    }
    else print(paste('Convergence was not reached'))
  }
  
#### Initial values for all parameters.
  
  a.factor<-6.5/var(Wij) # Attenuation factor # Considering 6.5 as the initial variance for true X)
  vare.initial<-6.5*(1-a.factor)/a.factor # 1.41
  theta.old<-c(-10.116,0.151,0.086,0.512,0.9,0.08,2.0,1.5,23.9,6.5,vare.initial)
  
#### Implication of MCEM function for MFUS data set.
 estimate<-MCEM(1000,0.1,theta.old)
## Calculating 95% confidence Interval.
  Est<-c(estimate[1],estimate[2],estimate[3],estimate[4],estimate[9],estimate[10],estimate[11],estimate[5],estimate[6],estimate[7],estimate[8])
  se<-c(estimate[12],estimate[13],estimate[14],estimate[15],estimate[20],estimate[21],estimate[22],estimate[16],estimate[17],estimate[18],estimate[19])
  LB=c()
  UB=c()
  for (i in 1:11)
  {
    LB[i]=Est[i]-1.96*se[i]
    UB[i]=Est[i]+1.96*se[i]
  }
  table_proposed<-data.frame(Est,se,LB,UB)
  return(table_proposed)
}

# Implementation of Proposed approach: (Section 6.1.1: Table 3: (Estimation of parameters))
set.seed(123)
Proposedresults=Proposed(n,q)
round(Proposedresults,3)



