############################ Approach 1: Naive 1 approach ############################################
## Function for Naive 1 approach:
######################################################################################################
date()
options(warn=-1)
naive1<-function(n,q,sigma)
{
# n= Individual size for data generation
# q= Number for follow-up for each individual
# sigma= Measurement error variation
# alpha=missing coefficients 
## Metropolis-Hasting (MH) Generation.
metro<- function(theta)
{
  ones<-matrix(rep(1,q),q,1) # qx1 vector of 1's.
  Ustar<-rnorm(n,0,sqrt(theta[5])) # initial simulation of random effects.
  z<-matrix(0,nrow=n*q,ncol=n) # nqxn, Design matrix
  for(j in 1:n)
  {
    z[(1+(j-1)*q):(q*j),j]=1
  }

    fixed<-theta[1]+theta[2]*Xij+theta[3]*Zij+theta[4]*Zistar # nx1 fixed effects in linear predictor.
    Fixed<-matrix(rep(fixed,1),n*q,1,byrow=T) # nqx1; copies of
    # fixed effects.
    umat<-matrix(rep(Ustar,1),n,1)# nx1; copies of previously simulated random effects.
    eta<-Fixed+z%*%umat # nqx1, copies of linear predictors based on previously simulated u's.
    pij<-exp(eta)/(1+exp(eta)) # probability of bernoulli trail.
    # simulate the true model
    y<-rbinom(n*q,1,pij)
    Y<-matrix(rep(y,1),n*q,1) # nqx1; copies of the response.
    density<-dbinom(Y,1,pij) # nqx1; Binomial density at the linear predictor.
    density<-matrix(density,n,q) # nxq; q copies of Binomial density.
    list(Ustar=Ustar,y=y,pij=pij)
}

## Generate covariates (X, Z and Zstar) based on information given in Section 5.
vi<-rnorm(n*q,0,0.5)
Eij<-rnorm(n*q,0,0.5)
Zij<-vi+Eij
#set.seed(49)
Zi=matrix(rep(sample(c(0,1), n, replace = TRUE),q),n,q)
Zistar=as.vector(matrix(Zi,ncol=1,byrow=T))
# Generate true covariate
mu_x<-1
ai<-rnorm(n*q,0,1)
eps_ij<-rnorm(n*q,0,1)
Xij<-mu_x+ai+eps_ij 
## Generate data from Error distribution of mixed normal distribution:    
components <- sample(1:2,prob=c(0.7,0.3),size=(n*q),replace=TRUE)
mus <- c(0,0)
sds <- c(0.8,10)
eij <- rnorm(n=(n*q),mean=mus[components],sd=sds[components])

# Generate surroagte using classical additive measurement error model
Wij<-Xij+eij 

### Expected-Maximization algorithm steps.
### Function for calculating the E and M-steps of MCEM algorithm.
MCEM<-function(M,tol,theta.o) 
{ 
  # M= Number of MC size.
  # tol= tolerance value for EM algorithm
  # theta.o= Intial values of parameters
  k<-0
  theta.k<-theta.o
  beta<-c(theta.k[1],theta.k[2],theta.k[3],theta.k[4])
  # Implementation of MH algorithm
  sample<-metro(theta.k) 
  y<-sample$y
  #y<-matrix(rep(y,1),n,q,byrow=T)
  U<-sample$Ustar
  yvector<-matrix(y,ncol=1,byrow=T)
  uvector2<-matrix(U,ncol=1,byrow=T)

## Likelihood function for fixed effects estimate
  ll<-function(beta)
  {
    beta0<-beta[1]
    beta1<-beta[2]
    beta2<-beta[3]
    beta3<-beta[4]
    U<-sample$Ustar
    fixed<-beta0+beta1*Wij+beta2*Zij+beta3*Zistar
    Fixed<-matrix(rep(fixed,1),n*q,1,byrow=T)
    z<-matrix(0,nrow=n*q,ncol=n) # nqxn, Design matrix.
    for(j in 1:n)
    {
      z[(1+(j-1)*q):(q*j),j]=1
    }
    uvector<-matrix(z%*%U,ncol=1,byrow=T)
    eta<-Fixed+uvector
    p<-exp(eta)/(1+exp(eta))
    d<-log(dbinom(yvector,1,p))
    sum(d)
  }
## Likelihood function for random effects estimate
  varu<-theta.k[5]
  ll.random<-function(varu)
  {
    U<-sample$Ustar
    z<-matrix(0,nrow=n*q,ncol=n) # nqxn, Design matrix.
    for(j in 1:n)
    {
      z[(1+(j-1)*q):(q*j),j]=1
    }
    uvector<-matrix(z%*%U,ncol=1,byrow=T)
    uvector2<-matrix(U,ncol=1,byrow=T)
    sum(log(dnorm(uvector,0,sqrt(varu))))
  }
  
## M step of EM algorithm.
  convergence <- F
  while(1-convergence){
    k <- k+1
#updated estimates for fixed parameters
    #beta<-optim(fn = ll, par = c(theta.k[1],theta.k[2],theta.k[3],theta.k[4]),lower = c(-4.88,-Inf, -Inf, -Inf),upper = c(-2,rep(1.5, 3)),control=list(fnscale=-1),hessian = TRUE)
    beta<-optim(fn = ll, par = c(theta.k[1],theta.k[2],theta.k[3],theta.k[4]),control=list(fnscale=-1),hessian = TRUE)
    beta.new<-beta$par
#note that this may give some values that are too large or not finite:
    #ifelse(beta.new<0 | beta.new>5,next,beta.new)
#updated estimates for random parameter
    varu.new<-optimize(ll.random,interval=c(0,5),maximum=TRUE)$maximum
    theta.new<-list(beta.new,varu.new)
    theta.new<-unlist(theta.new)
# Checking the convergence
    if (max(abs(c(theta.new[1]-theta.k[1],theta.new[2]-theta.k[2],theta.new[3]-theta.k[3],theta.new[4]-theta.k[4],theta.new[5]-theta.k[5])))>tol){
      theta.k <-theta.new
      beta<-c(theta.new[1],theta.new[2],theta.new[3],theta.new[4])
      varu<-theta.new[5]
    }
    else convergence <- T
  }
  if(convergence){
    mle=theta.new
    #print(mle)
##########    
###############################################################################   
    
    yvector<-matrix(y,ncol=1,byrow=T)
    yvector2<-yvector
    uvector2<-matrix(U,ncol=1,byrow=T)
    z<-matrix(0,nrow=n*q,ncol=n) # nqxn, Design matrix.
    for(j in 1:n)
    {
      z[(1+(j-1)*q):(q*j),j]=1
    }
    uvector<-matrix(z%*%U,ncol=1,byrow=T)
    beta<-c(mle[1],mle[2],mle[3],mle[4])
    fixed<-beta[1]+beta[2]*Wij+beta[3]*Zij+beta[4]*Zistar
    Fixed<-matrix(rep(fixed,1),n*q,1,byrow=T)
    eta<-Fixed+uvector
    p<-exp(eta)/(1+exp(eta))

    # First Derivatives of the log-likelihood function (Score function).

    S.beta0<-yvector2-p
    S.beta1<-(yvector2*Wij)-Wij*p
    S.beta2<-(yvector2*Zij)-Zij*p
    S.beta3<-(yvector2*Zistar)-Zistar*p
    S.sdu<-(-(1/mle[5])-(uvector^2/mle[5]^3))

    # Summing up over the repeated measures (over q follow-ups)

    b0<-matrix(S.beta0,q,n)
    b1<-matrix(S.beta1,q,n)
    b2<-matrix(S.beta2,q,n)
    b3<-matrix(S.beta3,q,n)
    sdu<-matrix(S.sdu,q,n)

    S.b0<-matrix(NA,nrow=n,ncol=1)
    S.b1<-matrix(NA,nrow=n,ncol=1)
    S.b2<-matrix(NA,nrow=n,ncol=1)
    S.b3<-matrix(NA,nrow=n,ncol=1)
    S.sdu<-matrix(sdu[1,],nrow=n,ncol=1)
    for (j in 1:n)
    {
      S.b0[j,]<-sum(b0[,j])
      S.b1[j,]<-sum(b1[,j])
      S.b2[j,]<-sum(b2[,j])
      S.b3[j,]<-sum(b3[,j])
    }
    ## Calculating the approximate Information matrix.
    info.mat<-matrix(rep(0,25),nrow=5)
    for (i in 1:n)
    {
      info.mat<-info.mat +c(S.b0[i],S.b1[i],S.b2[i],S.b3[i],S.sdu[i])%*%
        t(c(S.b0[i],S.b1[i],S.b2[i],S.b3[i],S.sdu[i]))
    }

    approx.info.mat<-info.mat
    det(approx.info.mat)
    var.cov<-solve(approx.info.mat)
    se<-as.vector(sqrt(diag(var.cov)))
    #estimates<-c(mle,se)
    estimates<-c(mle,se)
    return(estimates)
    #print(se.beta,se.varu)
  }
  else print(paste('Convergence was not reached'))
}

## Initial values for all parameters.
## calculating the intial value for random effect covariance based on Section 5.
sigma_sq=exp(.3+.5*Zi) # 0.3 and 0.5 are the intial values used in Section 5
varu.old<-round(mean(sigma_sq[,1]),2)
theta.old<-c(1,1,2,2,varu.old)

### Implication of MCEM function for 200 independent data sets.
set.seed(49)
estimate<-matrix(NA,200,10)
for (k in 1:200)
{
  t0=Sys.time()
  estimate[k,]<-MCEM(500,0.005,theta.old)
  print(Sys.time()-t0)
} 
## Calculating 95% confidence Interval and Coverage probability.
true.theta<-theta.old
mle<-cbind(estimate[,1],estimate[,2],estimate[,3],estimate[,4],estimate[,5])
se<-cbind(estimate[,6],estimate[,7],estimate[,8],estimate[,9],estimate[,10])
## Estimation of Bias, Root mean square error (RMSE).
theta<-apply(mle,2,mean)
bias<-round(theta-theta.old,5)
RMSE<-sqrt((apply(mle,2,var))+(bias^2)) # Root Mean square error.
SE<-apply(mle,2,sd)
CR<-c()
for (i in 1:5)
{
  ll=mle[,i]-1.96*SE[i]
  ul=mle[,i]+1.96*SE[i]
  coverage=((ll<=true.theta[i]) & (ul>=true.theta[i]))
  CR[i]<-mean(coverage) # Coverage of C.I for true theta 
}

table_naive1<-data.frame(bias,RMSE,CR)
return(table_naive1)
}
# Implementation of Naive 1 approach: (Section 5: Table 3 Naive 1)
set.seed(12345)
Naive1result=naive1(350,5,c(0.8,10));
round(Naive1result,4)


date()


















