
##### Approach 2: Naive 2 approach
### Function of MCEM algorithm for Naive 2 approach. 
date()
options(warn=-1)
naive2<-function(n,q,sigma){
# n= Sample size for data generation
# q= Number for follow-up for each individual
# sigma= Measurement error variations

## Metropolis-Hasting (MH) algorithm to generate data from random components as described in Section 4.
## Function for MH.
metropolis<- function (theta.k,M) 
  { 
 # M= Number of MC size.
 # theta.k= Initial values of parameters.
    Ustar<-matrix(0,nrow= M,ncol=n) # Mxn matrix to be filled with simulated u (random effects).
    xstar<-matrix(0,nrow= M,ncol=n*q) # Mxn matrix to be filled with simulated x (measurement error covariate).
    ap.x<-rep(0,n*q) # to store the acceptence probability for candidate for x.
    ap.u<-rep(0,n) # to store the acceptence probability for candidate for u.
    ones<-matrix(rep(1,q),q,1) # qx1 vector pf 1's.
    Ustar[1,]<-rnorm(n,0,sqrt(theta.k[5])) # initial simulation of random effects.
 ## Generate error covariate x based on information given in Section 5.
    mu_x<-1
    ai<-rnorm(n*q,0,1)
    eps_ij<-rnorm(n*q,0,1)
    xstar[1,]<-rnorm(n*q,1,sqrt(2))
    z<-matrix(0,nrow=n*q,ncol=n) # nqxn, Design matrix
    for(j in 1:n)
    {
      z[(1+(j-1)*q):(q*j),j]=1
    }
 ## Metropolis step.
    for (m in 2:M)
    { 
 ## Generate candidate vector for u and x.
      u.candidate<-rnorm(n,0,sqrt(theta.k[5])) 
      x.candidate<-matrix(rnorm(n*q,1,sqrt(2)),n,q)
 ## Calculate Denominator of Acceptance criterion vectors
      xmat<-matrix(xstar[m-1,],n,q) # nxq; copies of previously simulated x values.
      umat<-matrix(rep(Ustar[m-1,],1),n,1)# nx1; copies of previously simulated random effects.
      fixed.denom<-theta.k[1]+theta.k[2]*xmat+theta.k[3]*Zij+theta.k[4]*Zistar 
      # fixed effects in linear predictor.
      Fixed.denom<-matrix(rep(fixed.denom,1),n*q,1,byrow=T) # nqx1; copies of fixed effects.
      eta.denom<-Fixed.denom+z%*%umat # nqx1, copies of linear predictors based on previously simulated u's.
      pij.denom<-exp(eta.denom)/(1+exp(eta.denom)) # probability of bernoulli trail.
      
      y.denom<-rbinom(n*q,1,pij.denom) # nqx1; copies of the response.
      Y.denom<-matrix(rep(y.denom,1),n,q)
      
      density.y.denom<-dbinom(Y.denom,1,pij.denom) # nqx1; Binomial density at the linear predictor.
      density.Y.denom<-matrix(density.y.denom,n,q) # nxq; q copies of Binomial density.
      density.X.denom<-matrix(density.y.denom,n,q)
      apdenominator.u<-density.Y.denom%*%ones # nx1; jth element is the product of Binomial pmfs at previously simulatated values of uj.
      apdenominator.x<-density.X.denom
   ## Acceptance vector for random effects,u.
      umat2<-u.candidate # nx1; elements with new candidates.  
      fixed.num<-theta.k[1]+theta.k[2]*xmat+theta.k[3]*Zij+theta.k[4]*Zistar # nqx1 fixed effects in linear predictor.
      Fixed.num<-matrix(rep(fixed.num,1),n*q,1,byrow=T) # nqx1; copies of fixed effects.
      
      eta.num<-Fixed.num+z%*%umat2 # nqx1; copies of linear predictors based on previously simulated u's.
      pij.num<-exp(eta.num)/(1+exp(eta.num))
      
      y.num<-rbinom(n*q,1,pij.num)
      Y.num<-matrix(rep(y.num,1),n,q)
      density.y.num<-dbinom(Y.num,1,pij.num) # nqx1; jth column has Binomial pmfs with all u's equal 
      # to previously simulated values, except with uj replaced by new candidate.
      density.Y.num<-matrix(density.y.num,n,q) # nxq; q copies of Binomial density.
      apnumerator.u<-density.Y.num%*%ones # nx1; jth element is the product of n Binomial pmfs at previously simulatated values of u,except with uj  replaced by new candidate.
   
  ## Acceptance vector for x.
      xmat2<-x.candidate # nqx1; elements with new candidates.
      fixed.num<-theta.k[1]+theta.k[2]*xmat2+theta.k[3]*Zij+theta.k[4]*Zistar # nx1 fixed effects in linear predictor.
      Fixed.num<-matrix(rep(fixed.num,1),n*q,1,byrow=T) # nqx1; copies of fixed effects.
      
      eta.num<-Fixed.num+z%*%umat # nqx1; copies of linear predictors based on previously simulated u's.
      pij.num<-exp(eta.num)/(1+exp(eta.num))
      
      y.num<-rbinom(n*q,1,pij.num)
      Y.num<-matrix(rep(y.num,1),n,q)
      density.y.num<-dbinom(Y.num,1,pij.num) # nqx1; jth column has Binomial pmfs with all u's equal 
      # to previously simulated values, except with uj replaced by new candidate.
      density.Y.num<-matrix(density.y.num,n,q) # nxq; q copies of Binomial density.
      apnumerator.x<-density.Y.num
 
  ## Acceptance criterion for u and x.      
      ap.u<-ifelse(apnumerator.u<apdenominator.u,apnumerator.u/apdenominator.u,1)
      #acceptance probability of jth element for candidate uj.
      
      ap.x<-ifelse(apnumerator.x<apdenominator.x,apnumerator.x/apdenominator.x,1) 
      #acceptance probability of jth element for candidate xj.
      
      ## Acceptance rule.
      # Generate a bernoulli random variable with the candidate u acceptance probabilities.
      ber.u<-rbinom(n,1,ap.u) 
      ber.x<-rbinom(n*q,1,ap.x)
      Ustar[m,]<-ber.u*u.candidate+(1-ber.u)*Ustar[m-1,]  # nx1; new u is either previous u or new candidate.
      xstar[m,]<-ber.x*x.candidate+(1-ber.x)*xstar[m-1,] # nx1; new x is either previous u or new candidate.
    }
    burnin=500
    b=burnin+1
    index=b:M
    Ustar=Ustar[index,]
    xstar=xstar[index,]
    list(xstar=xstar,Ustar=Ustar)
  }

## Generate covariates (Z and Zstar) based on information given in Section 5.
vi<-rnorm(n*q,0,0.5)
Eij<-rnorm(n*q,0,0.5)
Zij<-vi+Eij
Zi=matrix(rep(sample(c(0,1), n, replace = TRUE),q),n,q)
Zistar=as.vector(matrix(Zi,ncol=1,byrow=T))

### Expected-Maximization algorithm steps.
### Function for calculating the E and M-steps of MCEM algorithm.
MCEM<-function(M,tol,theta.o)
{ 
  # M= Number of MC size.
  # tol= tolerance value for EM algorithm
  # theta.o= Intial values of parameters
  burnin=500
  Mm<-M-burnin
  k<-0
  theta.k<-theta.o
  beta<-c(theta.k[1],theta.k[2],theta.k[3],theta.k[4])
  varu<-theta.k[5]
## E-M step of EM algorithm. 
  convergence <- F
  while(1-convergence){
    k <- k+1
   # Implementation of MH algorithm
    sample<-metropolis(theta.k,M)
    U<-sample$Ustar # Mxn
    Xij<-t(sample$xstar) # nqxM
    z<-matrix(0,nrow=n*q,ncol=n) # nqxn, Design matrix.
    for(j in 1:n)
    {
      z[(1+(j-1)*q):(q*j),j]=1
    }

 ## Generate data from Error distribution of mixed normal distribution:    
    components <- sample(1:2,prob=c(0.7,0.3),size=(n*q),replace=TRUE)
    mus <- c(0,0)
    sds <- c(0.8,10)
    eij <- rnorm(n=(n*q),mean=mus[components],sd=sds[components])
 # Generate surroagte using classical additive measurement error model
    Wij<-Xij+eij
    fixed<-beta[1]+beta[2]*Wij+beta[3]*Zij+beta[4]*Zistar #nqxM
    Fixed<-matrix(rep(fixed,1),n*q*Mm,1,byrow=T)
    
    uvector<-matrix(z%*%t(U),ncol=1,byrow=T)
    eta<-Fixed+uvector
    p<-exp(eta)/(1+exp(eta))
    y<-rbinom(n*q*Mm,1,p) # copies of the response.
    
    yvector<-matrix(y,ncol=1,byrow=T) 
    uvector2<-matrix(U,ncol=1,byrow=T) # Mnx1
    
## Likelihood function for fixed effects estimate
    ll<-function(beta)
    {
      beta0<-beta[1]
      beta1<-beta[2]
      beta2<-beta[3]
      beta3<-beta[4]
      fixed<-beta0+beta1*Wij+beta2*Zij+beta3*Zistar
      Fixed<-matrix(rep(fixed,1),n*q*Mm,1,byrow=T)
      z<-matrix(0,nrow=n*q,ncol=n) # nqxn, Design matrix.
      for(j in 1:n)
      {
        z[(1+(j-1)*q):(q*j),j]=1
      }
      uvector<-matrix(z%*%t(U),ncol=1,byrow=T)
      eta<-Fixed+uvector
      p<-exp(eta)/(1+exp(eta))
      d<-log(dbinom(yvector,1,p))
      sum(d)/Mm
    }
 ## Likelihood function for random effects estimate   
    
    ll.random<-function(varu)
    {
      uvector2<-matrix(U,ncol=1,byrow=T)
      z<-matrix(0,nrow=n*q,ncol=n) # nqxn, Design matrix.
      for(j in 1:n)
      {
        z[(1+(j-1)*q):(q*j),j]=1
      }
      uvector<-matrix(z%*%t(U),ncol=1,byrow=T)
      sum(log(dnorm(uvector2,0,sqrt(varu))))/Mm
    }
# updated estimates for fixed parameters   
    beta<-optim(fn = ll, par = c(theta.k[1],theta.k[2],theta.k[3],theta.k[4]),control=list(fnscale=-1),hessian = TRUE)
    beta.new<-beta$par
# note that this may give some values that are too large or not finite:
    #ifelse(beta.new<0 | beta.new>5,next,beta.new)
# updated estimates for random parameter  
    varu.new<-optimize(ll.random,interval=c(0,5),maximum=TRUE)$maximum
    
    ## varu.new=sqrt(sum(uvector2^2))/M*n
    
    theta.new<-list(beta.new,varu.new)
    theta.new<-unlist(theta.new)
# Checking the convergence    
  if (max(abs(c(theta.new[1]-theta.k[1],theta.new[2]-theta.k[2],theta.new[3]-theta.k[3],theta.new[4]-theta.k[4],theta.new[5]-theta.k[5])))>tol){
      
    theta.k <-theta.new
    beta<-c(theta.new[1],theta.new[2],theta.new[3],theta.new[4])
    varu<-theta.new[5]
    }
    else convergence <- T
  }
  if(convergence){
    print(paste('Convergence was reached after',k,'iteration(s)'))
    mle=theta.new  
##### Calculation of standard errors for parameters.
    
    yvector2<-matrix(y,ncol=1,byrow=T)
    z<-matrix(0,nrow=n*q,ncol=n) # nqxn, Design matrix.
    for(j in 1:n)
    {
      z[(1+(j-1)*q):(q*j),j]=1
    }
    uvector2<-matrix(t(U),ncol=1,byrow=T)
    uvector<-matrix(z%*%t(U),ncol=1,byrow=T)
    beta<-c(mle[1],mle[2],mle[3],mle[4])
    fixed<-beta[1]+beta[2]*Wij+beta[3]*Zij+beta[4]*Zistar
    Fixed<-matrix(rep(fixed,1),n*q*Mm,1,byrow=T)
    eta<-Fixed+uvector
    p<-exp(eta)/(1+exp(eta))
# First Derivatives of the log-likelihood function (Score function).
    
    Wij.vec<-matrix(Wij,ncol=1,byrow=T)
    S.beta0<-yvector2-p
    S.beta1<-(yvector2*Wij.vec)-Wij.vec*p
    S.beta2<-(yvector2*Zij)-Zij*p
    S.beta3<-(yvector2*Zistar)-Zistar*p
    S.sdu<-(-(1/mle[5])-(uvector2^2/mle[5]^3))
 # Summing up over the repeated measures (over q follow-ups) 
    b0<-matrix(S.beta0,q,n*Mm)
    b1<-matrix(S.beta1,q,n*Mm)
    b2<-matrix(S.beta2,q,n*Mm)
    b3<-matrix(S.beta3,q,n*Mm)
    sdu<-matrix(S.sdu,q,n*Mm)
    
    S.b0<-matrix(NA,nrow=n*Mm,ncol=1)
    S.b1<-matrix(NA,nrow=n*Mm,ncol=1)
    S.b2<-matrix(NA,nrow=n*Mm,ncol=1)
    S.b3<-matrix(NA,nrow=n*Mm,ncol=1)
    S.sdu<-matrix(NA,nrow=n*Mm,ncol=1)
    
    for (j in 1:(n*Mm))
    {
      S.b0[j,]<-sum(b0[,j])
      S.b1[j,]<-sum(b1[,j])
      S.b2[j,]<-sum(b2[,j])
      S.b3[j,]<-sum(b3[,j])
      S.sdu[j,]<-sum(sdu[,j])
    }
  ## Calculating the approximate Information matrix.
    info.mat<-matrix(rep(0,25),nrow=5)
    
    for (i in 1:(n*Mm))
    {
      info.mat<-info.mat +c(S.b0[i],S.b1[i],S.b2[i],S.b3[i],S.sdu[i])%*%
        t(c(S.b0[i],S.b1[i],S.b2[i],S.b3[i],S.sdu[i]))
    }
    
    approx.info.mat<-1/Mm*info.mat
    det(approx.info.mat)
    var.cov<-solve(approx.info.mat)
    se<-as.vector(sqrt(diag(var.cov)))
    #print(se.5)
    estimates<-c(mle,se)
    return(estimates)
    #print(round(estimates),4)
  }
  else print(paste('Convergence was not reached'))
}
#### Initial values for all parameters.
## calculating the intial value for random effect covariance based on Section 5. 
sigma_sq=exp(.3+.5*Zi)  # 0.3 and 0.5 are the intial values used in Section 5.
varu.old<-round(mean(sigma_sq[,1]),2) ## Actaually its 1.75
theta.old<-c(1,1,2,2,varu.old)
### Implication of MCEM function for 200 independent data sets.
set.seed(49)
estimate<-matrix(NA,1,10)

  t0= Sys.time()
  estimate[1,]<-MCEM(1000,0.005,theta.old)
  print(Sys.time()-t0)

return(estimate)
}


date()