## Function for Proposed approach:
### Function of MCEM algorithm for Proposed approach.
date()
options(warn=-1)
Proposed<-function(n,q,sigma){
  # n= Sample size for data generation
  # q= Number for follow-up for each individual
  # sigma= Measurement error variation
  ## Metropolis-Hasting (MH) algorithm to generate data from random components as described in Section 4.
  ## Function for MH.
  metropolis<- function (theta.k,M) 
  {
    # M= Number of MC size.
    # theta.k= Initial values of parameters.
    Ustar<-matrix(0,nrow=M,ncol=n*q) # matrix to be filled with simulated u (random effects).
    xstar<-matrix(0,nrow=M,ncol=n*q) # matrix to be filled with simulated x (measurement error covariate).
    ap.x<-rep(0,n*q) # to store the acceptence probability for candidate for x.
    ap.u<-rep(0,n*q) # to store the acceptence probability for candidate for u.
    ones<-matrix(rep(1,q),q,1) # qx1 vector pf 1's.
    ## Function for generating random effects by proposed approach following Section 3.
    ucandid<-function(n,q){
      u.all=matrix(NA,n,q)
      # Indicator variable to generate GAR and IV parameters based on information given in Section 5.
      I=matrix(NA,q,q)  
      for (i in 1:n){
        for (j in 1:q){
          for (t in (1:q)){
            #I[j,t]=ifelse((abs(j-t)==1),1,0)
            I[j,t]=if(abs(j-t)==1) 1 else 0
          }
        }
        phi=theta.k[5]*I+theta.k[6]*I*Zi[i,]
        T.hat=phi+diag(1,q,q)
        T.hat[upper.tri(T.hat)] <-0
        T.hat[col(T.hat)<row(T.hat)] <- -1*T.hat[col(T.hat)<row(T.hat)]
        sigma_sq=sqrt(exp(theta.k[7]+theta.k[8]*Zi[i,]))
        D.hat=diag(sigma_sq,q,q)
        Sigma=solve(T.hat)%*%D.hat%*%t(solve(T.hat))
        mean=rep(0,q)
        ## Load relevant R library
        #require(mvtnorm)
        library(mvtnorm)
        u.all[i,]=rmvnorm(1,mean,Sigma)
      }
      U.all=matrix(t(u.all),ncol=1,byrow=T)
      return(list(U.all=U.all,u.all=u.all,I=I))
    }
    
    Ustar[1,]<-ucandid(n,q)$U.all # initial simulation of random effects.
    ## Generate error covariate x based on information given in Section 5.
    mu_x<-1
    ai<-rnorm(n*q,0,1)
    eps_ij<-rnorm(n*q,0,1)
    xstar[1,]<-rnorm(n*q,1,sqrt(2))
    ## Metropolis step.
    for (m in 2:M)
    {
      ## Generate candidate vector for u and x.
      u.candidate<-ucandid(n,q)$U.all 
      x.candidate<-matrix(rnorm(n*q,1,sqrt(2)),n,q)
      ## Calculate Denominator of Acceptance criterion vectors
      xmat<-matrix(xstar[m-1,],n,q) # copies of previously simulated x.
      umat<-matrix(Ustar[m-1,],n,q) # copies of previously simulated random effects.
      umat<-matrix(umat,n*q,1,byrow=T)
      #fixed effects in linear predictor.
      fixed.denom<-theta.k[1]+theta.k[2]*xmat+theta.k[3]*Zij+theta.k[4]*Zistar
      Fixed.denom<-matrix(rep(fixed.denom,1),n*q,1,byrow=T) # nqx1; copies of fixed effects.
      eta.denom<-Fixed.denom+umat # copies of linear predictors based on previously simulated u's.
      pij.denom<-exp(eta.denom)/(1+exp(eta.denom)) # probability of bernoulli trail.
      
      y.denom<-rbinom(n*q,1,pij.denom)
      Y.denom<-matrix(rep(y.denom,1),n,q) # nqx1; copies of the response.
      
      density.y.denom<-dbinom(Y.denom,1,pij.denom) # Binomial density at the linear predictor.
      density.Y.denom<-matrix(density.y.denom,n,q) # nxq; q copies of Binomial density.
      density.X.denom<-matrix(density.y.denom,n,q)
      apdenominator.u<-density.Y.denom # jth element is the product of Binomial pmfs at previously simulatated values of uj. 
      apdenominator.x<-density.X.denom
      
      ## Acceptance vector for random effects,u.
      umat2<-u.candidate # elements with new candidates.  
      fixed.num<-theta.k[1]+theta.k[2]*xmat+theta.k[3]*Zij+theta.k[4]*Zistar # nx1 fixed effects in linear predictor.
      Fixed.num<-matrix(rep(fixed.num,1),n*q,1,byrow=T) # copies of fixed effects.
      eta.num<-Fixed.num+umat2 # copies of linear predictors based on previously simulated u's.
      pij.num<-exp(eta.num)/(1+exp(eta.num))
      y.num<-rbinom(n*q,1,pij.num)
      Y.num<-matrix(rep(y.num,1),n,q) # copies of the response.
      density.y.num<-dbinom(Y.num,1,pij.num) # nqx1; jth column has Binomial pmfs with all u's equal 
      # to previously simulated values, except with uj replaced by new candidate.
      density.Y.num<-matrix(density.y.num,n,q) # nxq; q copies of Binomial density.
      apnumerator.u<-density.Y.num
      
      ## Acceptance vector for x.   
      xmat2<-x.candidate # elements with new candidates.   
      fixed.num<-theta.k[1]+theta.k[2]*xmat2+theta.k[3]*Zij+theta.k[4]*Zistar # nx1 fixed effects in linear predictor.
      Fixed.num<-matrix(rep(fixed.num,1),n*q,1,byrow=T) # copies of fixed effects.
      eta.num<-Fixed.num+umat # copies of linear predictors based on previously simulated u's.
      pij.num<-exp(eta.num)/(1+exp(eta.num))
      y.num<-rbinom(n*q,1,pij.num)
      
      Y.num<-matrix(rep(y.num,1),n,q) # copies of the response.
      density.y.num<-dbinom(Y.num,1,pij.num) # jth column has Binomial pmfs with all u's equal 
      # to previously simulated values, except with uj replaced by new candidate.
      density.Y.num<-matrix(density.y.num,n,q) # nxq; q copies of Binomial density.  
      apnumerator.x<-density.Y.num
      
      ## Acceptance criterion for u and x. 
      ap.u<-ifelse(apnumerator.u<apdenominator.u,apnumerator.u/apdenominator.u,1)  
      #acceptance probability of jth element for candidate uj.
      
      ap.x<-ifelse(apnumerator.x<apdenominator.x,apnumerator.x/apdenominator.x,1) 
      #acceptance probability of jth element for candidate xj.
      
      ## Acceptance rule.
      # Generate a bernoulli random variable with the candidate u acceptance probabilities.
      ber.u<-rbinom(n*q,1,ap.u) 
      ber.x<-rbinom(n*q,1,ap.x)
      Ustar[m,]<-ber.u*u.candidate+(1-ber.u)*Ustar[m-1,]  # nx1; new u is either previous u 
      # or new candidate.
      xstar[m,]<-ber.x*x.candidate+(1-ber.x)*xstar[m-1,] # nx1; new x is either previous u 
      # or new candidate.
    }
    burnin=500
    b=burnin+1
    index=b:M
    Ustar=Ustar[index,]
    xstar=xstar[index,]
    list(xstar=xstar,Ustar=Ustar)
  }
  
  ## Generate covariates (Z and Zstar) based on information given in Section 5.
  vi<-rnorm(n*q,0,0.5)
  Eij<-rnorm(n*q,0,0.5)
  Zij<-vi+Eij 
  Zi=matrix(rep(sample(c(0,1), n, replace = TRUE),q),n,q)
  Zistar=as.vector(matrix(Zi,ncol=1,byrow=T))
  
  ### Expected-Maximization algorithm steps.
  ## Indicator variable to generate GAR and IV parameters based on information given in Section 5.
  I=matrix(NA,q,q)
  for (j in 1:q)
  {
    for (t in (1:q))
    {
      I[j,t]=if(abs(j-t)==1) 1 else 0
    }
  }
  ### Function for calculating the E and M-steps of Proposed algorithm.
  MCEM<-function(M,tol,theta.o) # M is the number of MC size.
  { 
    # M= Number of MC size.
    # tol= tolerance value for EM algorithm
    # theta.o= Intial values of parameters
    burnin=500
    Mm<-M-burnin
    k<-0
    theta.k<-theta.o
    beta<-c(theta.k[1],theta.k[2],theta.k[3],theta.k[4])
    varu<-c(theta.k[5],theta.k[6],theta.k[7],theta.k[8])
    ## E-M step of EM algorithm.   
    convergence <- F
    while(1-convergence){
      k <- k+1
      sample<-metropolis(theta.k,M)
      U<-sample$Ustar # Mxn
      Xij<-t(sample$xstar) # nqxM
   ## Generate data from Error distribution of mixed normal distribution:    
      components <- sample(1:2,prob=c(0.7,0.3),size=(n*q),replace=TRUE)
      mus <- c(0,0)
      sds <- c(0.8,10)
      eij <- rnorm(n=(n*q),mean=mus[components],sd=sds[components])
      
      # Generate surroagte using classical additive measurement error model
      Wij<-Xij+eij 
      fixed<-beta[1]+beta[2]*Wij+beta[3]*Zij+beta[4]*Zistar #nqxM
      Fixed<-matrix(rep(fixed,1),n*q*Mm,1,byrow=T)
      
      uvector2<-matrix(t(U),ncol=1,byrow=T)
      eta<-Fixed+uvector2
      p<-exp(eta)/(1+exp(eta))
      y<-rbinom(n*q*Mm,1,p) # copies of the response.
      yvector<-matrix(y,ncol=1,byrow=T) 
      
      ## Function for estimating fixed effect parameters.
      ll<-function(beta)
      {
        beta0<-beta[1]
        beta1<-beta[2]
        beta2<-beta[3]
        beta3<-beta[4]
        fixed<-beta0+beta1*Wij+beta2*Zij+beta3*Zistar
        #fixed<-x%*%beta
        Fixed<-matrix(rep(fixed,1),n*q*Mm,1,byrow=T)
        eta<-Fixed+uvector2
        p<-exp(eta)/(1+exp(eta))
        d<-log(dbinom(yvector,1,p))
        sum(d)/Mm
      }
      ## Function for estimating random effect parameters.  
      ll.random<-function(varu)
      {
        delta0<-varu[1] # GAR parameters for random effects cov. matrix 
        delta1<-varu[2]
        lambda0<-varu[3] # IV parameters for random effects cov. matrix
        lambda1<-varu[4]
        u.all=matrix(NA,n,q)
        dd=matrix(NA,n,q)
        logdd=matrix(NA,n,q)
        Sigma=matrix(NA,n,q*q)
        for (i in 1:n){
          phi=delta0*I+delta1*I*Zi[i,]
          T.hat=phi+diag(1,q,q)
          T.hat[upper.tri(T.hat)]<-0
          T.hat[col(T.hat)<row(T.hat)] <- -1*(T.hat[col(T.hat)<row(T.hat)])
          sigma_sq=sqrt(exp(lambda0+lambda1*Zi[i,]))
          D.hat=diag(sigma_sq,q,q)
          Sigma=t(solve(T.hat))%*%D.hat%*%solve(T.hat)
          mean=rep(0,q)
          library(mvtnorm)
          u.all[i,]=rmvnorm(1,mean,Sigma)
          dd=dmvnorm(u.all,mean,Sigma,log=TRUE)
        }
        sum(dd)/n
      }
      # updated estimates for fixed parameters.   
      fixed<-optim(fn = ll, par = c(theta.k[1],theta.k[2],theta.k[3],theta.k[4]),method = "BFGS",control=list(fnscale=-1),hessian = TRUE)
      # note that this may give some values that are too large or not finite:
      ifelse(fixed$par<0 | fixed$par>5,next,fixed$par)
      beta.new<-fixed$par
      # updated estimates for random effect cov. matrix parameters.     
      random<-optim(fn=ll.random, par = c(theta.k[5],theta.k[6],theta.k[7],theta.k[8]),method = "L-BFGS-B",
                    lower=c(.01,.01,.01,.01),upper=c(3,3,3,3),control=list(fnscale=-1),hessian = TRUE)
      ifelse(random$par<0 | random$par>5,next,random$par)
      varu.new<-random$par
      theta.new<-list(beta.new,varu.new)
      theta.new<-unlist(theta.new)
   # Checking the convergence.      
      if (max(abs(c(theta.new[1]-theta.k[1],theta.new[2]-theta.k[2],theta.new[3]-theta.k[3],theta.new[4]-theta.k[4],theta.new[5]-theta.k[5],theta.new[6]-theta.k[6],theta.new[7]-theta.k[7],theta.new[8]-theta.k[8])))>tol){
        theta.k <-theta.new      
        beta<-c(theta.new[1],theta.new[2],theta.new[3],theta.new[4])
        varu<-c(theta.new[5],theta.new[6],theta.new[7],theta.new[8])
      }
      else convergence <- T
    }
    if(convergence){
      print(paste('Convergence was reached after',k,'iteration(s)'))
      mle=theta.new
      
      ### Calculation of standard errors for parameters from "optim" function.
      # NEGATIVE of the hessian is the "fishers observed information"
      I.fixed<-solve(-fixed$hessian) 
      se.fixed<-sqrt(diag(I.fixed))
      
      I.rand<-solve(-random$hessian)
      se.rand<-sqrt(abs(diag(I.rand)))
      
      se<-c(se.fixed,se.rand)  
      
      estimates<-c(mle,se)
      return(estimates)
    }
    else print(paste('Convergence was not reached'))
  }
  
  #### Initial values for all parameters.
  theta.old<-c(1,1,2,2,0.5,0.3,0.1,0.2)
  
  ### Implication of MCEM function for 200 independent data sets.
  set.seed(49)
  estimate<-matrix(NA,1,16)
   t0=Sys.time()
   estimate[1,]<-MCEM(1000,0.05,theta.old)
   print(Sys.time()-t0)
  return(estimate)  
}


date()










    